@@ -1,15 +1,7 @@
 /*
- * Copyright (C) by Roeland Jago Douma <rullzer@owncloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2019 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2015 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "sharemanager.h"
@@ -17,6 +9,8 @@
 #include "account.h"
 #include "folderman.h"
 #include "accountstate.h"
+#include "clientsideencryption.h"
+#include "updatee2eefolderusersmetadatajob.h"
 
 #include <QUrl>
 #include <QJsonDocument>
@@ -30,16 +24,16 @@ namespace OCC {
 /**
  * When a share is modified, we need to tell the folders so they can adjust overlay icons
  */
-static void updateFolder(const AccountPtr &account, const QString &path)
+static void updateFolder(const AccountPtr &account, QStringView path)
 {
-    foreach (Folder *f, FolderMan::instance()->map()) {
+    for (auto f : std::as_const(FolderMan::instance()->map())) {
         if (f->accountState()->account() != account)
             continue;
         auto folderPath = f->remotePath();
         if (path.startsWith(folderPath) && (path == folderPath || folderPath.endsWith('/') || path[folderPath.size()] == '/')) {
             // Workaround the fact that the server does not invalidate the etags of parent directories
             // when something is shared.
-            auto relative = path.midRef(f->remotePathTrailingSlash().length());
+            auto relative = path.mid(f->remotePathTrailingSlash().length());
             f->journalDb()->schedulePathForRemoteDiscovery(relative.toString());
 
             // Schedule a sync so it can update the remote permission flag and let the socket API
@@ -52,16 +46,18 @@ static void updateFolder(const AccountPtr &account, const QString &path)
 
 Share::Share(AccountPtr account,
     const QString &id,
-    const QString &uidowner,
+    const QString &uidOwner,
+    const QString &uidFileOwner,
     const QString &ownerDisplayName,
     const QString &path,
     const ShareType shareType,
     bool isPasswordSet,
     const Permissions permissions,
-    const QSharedPointer<Sharee> shareWith)
+    const ShareePtr shareWith)
     : _account(account)
     , _id(id)
-    , _uidowner(uidowner)
+    , _uidOwner(uidOwner)
+    , _uidFileOwner(uidFileOwner)
     , _ownerDisplayName(ownerDisplayName)
     , _path(path)
     , _shareType(shareType)
@@ -88,7 +84,12 @@ QString Share::getId() const
 
 QString Share::getUidOwner() const
 {
-    return _uidowner;
+    return _uidOwner;
+}
+
+QString Share::getUidFileOwner() const
+{
+    return _uidFileOwner;
 }
 
 QString Share::getOwnerDisplayName() const
@@ -101,7 +102,7 @@ Share::ShareType Share::getShareType() const
     return _shareType;
 }
 
-QSharedPointer<Sharee> Share::getShareWith() const
+ShareePtr Share::getShareWith() const
 {
     return _shareWith;
 }
@@ -146,6 +147,12 @@ void Share::deleteShare()
     job->deleteShare(getId());
 }
 
+bool Share::isShareTypeUserGroupEmailRoomOrRemote(const ShareType type)
+{
+    return (type == Share::TypeUser || type == Share::TypeGroup || type == Share::TypeEmail || type == Share::TypeRoom
+        || type == Share::TypeRemote);
+}
+
 void Share::slotDeleted()
 {
     updateFolder(_account, _path);
@@ -187,7 +194,8 @@ QDate LinkShare::getExpireDate() const
 
 LinkShare::LinkShare(AccountPtr account,
     const QString &id,
-    const QString &uidowner,
+    const QString &uidOwner,
+    const QString &uidFileOwner,
     const QString &ownerDisplayName,
     const QString &path,
     const QString &name,
@@ -195,12 +203,18 @@ LinkShare::LinkShare(AccountPtr account,
     Permissions permissions,
     bool isPasswordSet,
     const QUrl &url,
-    const QDate &expireDate)
-    : Share(account, id, uidowner, ownerDisplayName, path, Share::TypeLink, isPasswordSet, permissions)
+    const QDate &expireDate,
+    const QString &note,
+    const QString &label,
+    const bool hideDownload)
+    : Share(account, id, uidOwner, uidFileOwner, ownerDisplayName, path, Share::TypeLink, isPasswordSet, permissions)
     , _name(name)
     , _token(token)
+    , _note(note)
     , _expireDate(expireDate)
     , _url(url)
+    , _label(label)
+    , _hideDownload(hideDownload)
 {
 }
 
@@ -224,20 +238,24 @@ QString LinkShare::getNote() const
     return _note;
 }
 
+QString LinkShare::getLabel() const
+{
+    return _label;
+}
+
+bool LinkShare::getHideDownload() const
+{
+    return _hideDownload;
+}
+
 void LinkShare::setName(const QString &name)
 {
-    auto *job = new OcsShareJob(_account);
-    connect(job, &OcsShareJob::shareJobFinished, this, &LinkShare::slotNameSet);
-    connect(job, &OcsJob::ocsError, this, &LinkShare::slotOcsError);
-    job->setName(getId(), name);
+    createShareJob(&LinkShare::slotNameSet)->setName(getId(), name);
 }
 
 void LinkShare::setNote(const QString &note)
 {
-    auto *job = new OcsShareJob(_account);
-    connect(job, &OcsShareJob::shareJobFinished, this, &LinkShare::slotNoteSet);
-    connect(job, &OcsJob::ocsError, this, &LinkShare::slotOcsError);
-    job->setNote(getId(), note);
+    createShareJob(&LinkShare::slotNoteSet)->setNote(getId(), note);
 }
 
 void LinkShare::slotNoteSet(const QJsonDocument &, const QVariant &note)
@@ -253,10 +271,25 @@ QString LinkShare::getToken() const
 
 void LinkShare::setExpireDate(const QDate &date)
 {
+    createShareJob(&LinkShare::slotExpireDateSet)->setExpireDate(getId(), date);
+}
+
+void LinkShare::setLabel(const QString &label)
+{
+    createShareJob(&LinkShare::slotLabelSet)->setLabel(getId(), label);
+}
+
+void LinkShare::setHideDownload(const bool hideDownload)
+{
+    createShareJob(&LinkShare::slotHideDownloadSet)->setHideDownload(getId(), hideDownload);
+}
+
+template <typename LinkShareSlot>
+OcsShareJob *LinkShare::createShareJob(const LinkShareSlot slotFunction) {
     auto *job = new OcsShareJob(_account);
-    connect(job, &OcsShareJob::shareJobFinished, this, &LinkShare::slotExpireDateSet);
+    connect(job, &OcsShareJob::shareJobFinished, this, slotFunction);
     connect(job, &OcsJob::ocsError, this, &LinkShare::slotOcsError);
-    job->setExpireDate(getId(), date);
+    return job;
 }
 
 void LinkShare::slotExpireDateSet(const QJsonDocument &reply, const QVariant &value)
@@ -281,22 +314,41 @@ void LinkShare::slotNameSet(const QJsonDocument &, const QVariant &value)
     emit nameSet();
 }
 
+void LinkShare::slotLabelSet(const QJsonDocument &, const QVariant &label)
+{
+    if (_label != label.toString()) {
+        _label = label.toString();
+        emit labelSet();
+    }
+}
+
+void LinkShare::slotHideDownloadSet(const QJsonDocument &jsonDoc, const QVariant &hideDownload)
+{
+    Q_UNUSED(jsonDoc);
+    if (!hideDownload.isValid()) {
+        return;
+    }
+    _hideDownload = hideDownload.toBool();
+    emit hideDownloadSet();
+}
+
 UserGroupShare::UserGroupShare(AccountPtr account,
     const QString &id,
-    const QString &owner,
+    const QString &uidOwner,
+    const QString &uidFileOwner,
     const QString &ownerDisplayName,
     const QString &path,
     const ShareType shareType,
     bool isPasswordSet,
     const Permissions permissions,
-    const QSharedPointer<Sharee> shareWith,
+    const ShareePtr shareWith,
     const QDate &expireDate,
     const QString &note)
-    : Share(account, id, owner, ownerDisplayName, path, shareType, isPasswordSet, permissions, shareWith)
+    : Share(account, id, uidOwner, uidFileOwner, ownerDisplayName, path, shareType, isPasswordSet, permissions, shareWith)
     , _note(note)
     , _expireDate(expireDate)
 {
-    Q_ASSERT(shareType == TypeUser || shareType == TypeGroup || shareType == TypeEmail);
+    Q_ASSERT(Share::isShareTypeUserGroupEmailRoomOrRemote(shareType));
     Q_ASSERT(shareWith);
 }
 
@@ -326,6 +378,11 @@ QDate UserGroupShare::getExpireDate() const
 
 void UserGroupShare::setExpireDate(const QDate &date)
 {
+    if (_expireDate == date) {
+        emit expireDateSet();
+        return;
+    }
+
     auto *job = new OcsShareJob(_account);
     connect(job, &OcsShareJob::shareJobFinished, this, &UserGroupShare::slotExpireDateSet);
     connect(job, &OcsJob::ocsError, this, &UserGroupShare::slotOcsError);
@@ -364,6 +421,14 @@ void ShareManager::createLinkShare(const QString &path,
     job->createLinkShare(path, name, password);
 }
 
+void ShareManager::createSecureFileDropShare(const QString &path, const QString &name, const QString &password)
+{
+    const auto createShareJob = new OcsShareJob(_account);
+    connect(createShareJob, &OcsShareJob::shareJobFinished, this, &ShareManager::slotLinkShareCreated);
+    connect(createShareJob, &OcsJob::ocsError, this, &ShareManager::slotOcsError);
+    createShareJob->createSecureFileDropLinkShare(path, name, password);
+}
+
 void ShareManager::slotLinkShareCreated(const QJsonDocument &reply)
 {
     QString message;
@@ -397,10 +462,10 @@ void ShareManager::createShare(const QString &path,
     auto job = new OcsShareJob(_account);
     connect(job, &OcsJob::ocsError, this, &ShareManager::slotOcsError);
     connect(job, &OcsShareJob::shareJobFinished, this,
-        [=](const QJsonDocument &reply) {
+        [=, this](const QJsonDocument &reply) {
             // Find existing share permissions (if this was shared with us)
-            Share::Permissions existingPermissions = SharePermissionDefault;
-            foreach (const QJsonValue &element, reply.object()["ocs"].toObject()["data"].toArray()) {
+            Share::Permissions existingPermissions = SharePermissionAll;
+            for (const auto &element : reply.object()["ocs"].toObject()["data"].toArray()) {
                 auto map = element.toObject();
                 if (map["file_target"] == path)
                     existingPermissions = Share::Permissions(map["permissions"].toInt());
@@ -409,10 +474,10 @@ void ShareManager::createShare(const QString &path,
             // Limit the permissions we request for a share to the ones the item
             // was shared with initially.
             auto validPermissions = desiredPermissions;
-            if (validPermissions == SharePermissionDefault) {
+            if (validPermissions == SharePermissionAll) {
                 validPermissions = existingPermissions;
             }
-            if (existingPermissions != SharePermissionDefault) {
+            if (existingPermissions != SharePermissionAll) {
                 validPermissions &= existingPermissions;
             }
 
@@ -424,12 +489,46 @@ void ShareManager::createShare(const QString &path,
     job->getSharedWithMe();
 }
 
+void ShareManager::createE2EeShareJob(const QString &fullRemotePath,
+                                      const ShareePtr sharee,
+                                      const Share::Permissions permissions,
+                                      const QString &password)
+{
+    Folder *folder = nullptr;
+    for (const auto &f : FolderMan::instance()->map()) {
+        if (f->accountState()->account() != _account) {
+            continue;
+        }
+        folder = f;
+    }
+
+    if (!folder) {
+        emit serverError(0, "Failed creating share");
+        return;
+    }
+
+    Q_ASSERT(folder->remotePath() == QStringLiteral("/") ||
+        Utility::noLeadingSlashPath(fullRemotePath).startsWith(Utility::noLeadingSlashPath(Utility::noTrailingSlashPath(folder->remotePath()))));
+
+    const auto createE2eeShareJob = new UpdateE2eeFolderUsersMetadataJob(_account,
+                                                                         folder->journalDb(),
+                                                                         folder->remotePath(),
+                                                                         UpdateE2eeFolderUsersMetadataJob::Add,
+                                                                         fullRemotePath,
+                                                                         sharee->shareWith(),
+                                                                         QSslCertificate{},
+                                                                         this);
+
+    createE2eeShareJob->setUserData({sharee, permissions, password});
+    connect(createE2eeShareJob, &UpdateE2eeFolderUsersMetadataJob::finished, this, &ShareManager::slotCreateE2eeShareJobFinised);
+    createE2eeShareJob->start();
+}
 
 void ShareManager::slotShareCreated(const QJsonDocument &reply)
 {
     //Parse share
     auto data = reply.object().value("ocs").toObject().value("data").toObject();
-    QSharedPointer<Share> share(parseShare(data));
+    SharePtr share(parseShare(data));
 
     emit shareCreated(share);
 
@@ -438,45 +537,66 @@ void ShareManager::slotShareCreated(const QJsonDocument &reply)
 
 void ShareManager::fetchShares(const QString &path)
 {
-    auto *job = new OcsShareJob(_account);
+    const auto job = new OcsShareJob(_account);
     connect(job, &OcsShareJob::shareJobFinished, this, &ShareManager::slotSharesFetched);
     connect(job, &OcsJob::ocsError, this, &ShareManager::slotOcsError);
     job->getShares(path);
 }
 
-void ShareManager::slotSharesFetched(const QJsonDocument &reply)
+void ShareManager::fetchSharedWithMe(const QString &path)
 {
+    const auto sharedWithMeJob = new OcsShareJob(_account);
+    connect(sharedWithMeJob, &OcsShareJob::shareJobFinished, this, &ShareManager::slotSharedWithMeFetched);
+    connect(sharedWithMeJob, &OcsJob::ocsError, this, &ShareManager::slotOcsError);
+    sharedWithMeJob->getSharedWithMe(path);
+}
+
+const QList<SharePtr> ShareManager::parseShares(const QJsonDocument &reply) const
+{
+    qDebug() << reply;
     auto tmpShares = reply.object().value("ocs").toObject().value("data").toArray();
     const QString versionString = _account->serverVersion();
     qCDebug(lcSharing) << versionString << "Fetched" << tmpShares.count() << "shares";
 
-    QList<QSharedPointer<Share>> shares;
+    QList<SharePtr> shares;
 
-    foreach (const auto &share, tmpShares) {
+    for (const auto &share : tmpShares) {
         auto data = share.toObject();
 
         auto shareType = data.value("share_type").toInt();
 
-        QSharedPointer<Share> newShare;
+        SharePtr newShare;
 
         if (shareType == Share::TypeLink) {
             newShare = parseLinkShare(data);
-        } else if (shareType == Share::TypeGroup || shareType == Share::TypeUser || shareType == Share::TypeEmail) {
+        } else if (Share::isShareTypeUserGroupEmailRoomOrRemote(static_cast <Share::ShareType>(shareType))) {
             newShare = parseUserGroupShare(data);
         } else {
             newShare = parseShare(data);
         }
 
-        shares.append(QSharedPointer<Share>(newShare));
+        shares.append(SharePtr(newShare));
     }
 
     qCDebug(lcSharing) << "Sending " << shares.count() << "shares";
+    return shares;
+}
+
+void ShareManager::slotSharesFetched(const QJsonDocument &reply)
+{
+    const auto shares = parseShares(reply);
     emit sharesFetched(shares);
 }
 
-QSharedPointer<UserGroupShare> ShareManager::parseUserGroupShare(const QJsonObject &data)
+void ShareManager::slotSharedWithMeFetched(const QJsonDocument &reply)
 {
-    QSharedPointer<Sharee> sharee(new Sharee(data.value("share_with").toString(),
+    const auto shares = parseShares(reply);
+    emit sharedWithMeFetched(shares);
+}
+
+QSharedPointer<UserGroupShare> ShareManager::parseUserGroupShare(const QJsonObject &data) const
+{
+    ShareePtr sharee(new Sharee(data.value("share_with").toString(),
         data.value("share_with_displayname").toString(),
         static_cast<Sharee::Type>(data.value("share_type").toInt())));
 
@@ -493,6 +613,7 @@ QSharedPointer<UserGroupShare> ShareManager::parseUserGroupShare(const QJsonObje
     return QSharedPointer<UserGroupShare>(new UserGroupShare(_account,
         data.value("id").toVariant().toString(), // "id" used to be an integer, support both
         data.value("uid_owner").toVariant().toString(),
+        data.value("uid_file_owner").toVariant().toString(),
         data.value("displayname_owner").toVariant().toString(),
         data.value("path").toString(),
         static_cast<Share::ShareType>(data.value("share_type").toInt()),
@@ -503,7 +624,7 @@ QSharedPointer<UserGroupShare> ShareManager::parseUserGroupShare(const QJsonObje
         note));
 }
 
-QSharedPointer<LinkShare> ShareManager::parseLinkShare(const QJsonObject &data)
+QSharedPointer<LinkShare> ShareManager::parseLinkShare(const QJsonObject &data) const
 {
     QUrl url;
 
@@ -524,10 +645,16 @@ QSharedPointer<LinkShare> ShareManager::parseLinkShare(const QJsonObject &data)
     if (data.value("expiration").isString()) {
         expireDate = QDate::fromString(data.value("expiration").toString(), "yyyy-MM-dd 00:00:00");
     }
+    
+    QString note;
+    if (data.value("note").isString()) {
+        note = data.value("note").toString();
+    }
 
     return QSharedPointer<LinkShare>(new LinkShare(_account,
         data.value("id").toVariant().toString(), // "id" used to be an integer, support both
         data.value("uid_owner").toString(),
+        data.value("uid_file_owner").toString(),
         data.value("displayname_owner").toString(),
         data.value("path").toString(),
         data.value("name").toString(),
@@ -535,18 +662,22 @@ QSharedPointer<LinkShare> ShareManager::parseLinkShare(const QJsonObject &data)
         (Share::Permissions)data.value("permissions").toInt(),
         data.value("share_with").isString(), // has password?
         url,
-        expireDate));
+        expireDate,
+        note,
+        data.value("label").toString(),
+        data.value("hide_download").toInt() == 1));
 }
 
-QSharedPointer<Share> ShareManager::parseShare(const QJsonObject &data)
+SharePtr ShareManager::parseShare(const QJsonObject &data) const
 {
-    QSharedPointer<Sharee> sharee(new Sharee(data.value("share_with").toString(),
+    ShareePtr sharee(new Sharee(data.value("share_with").toString(),
         data.value("share_with_displayname").toString(),
         (Sharee::Type)data.value("share_type").toInt()));
 
-    return QSharedPointer<Share>(new Share(_account,
+    return SharePtr(new Share(_account,
         data.value("id").toVariant().toString(), // "id" used to be an integer, support both
         data.value("uid_owner").toVariant().toString(),
+        data.value("uid_file_owner").toVariant().toString(),
         data.value("displayname_owner").toVariant().toString(),
         data.value("path").toString(),
         (Share::ShareType)data.value("share_type").toInt(),
@@ -559,4 +690,28 @@ void ShareManager::slotOcsError(int statusCode, const QString &message)
 {
     emit serverError(statusCode, message);
 }
+
+
+void ShareManager::slotCreateE2eeShareJobFinised(int statusCode, const QString &message)
+{
+    const auto job = qobject_cast<UpdateE2eeFolderUsersMetadataJob *>(sender());
+    Q_ASSERT(job);
+    if (!job) {
+        qCWarning(lcUserGroupShare) << "slotCreateE2eeShareJobFinised must be called by UpdateE2eeShareMetadataJob::finished signal!";
+        return;
+    }
+    disconnect(job, &UpdateE2eeFolderUsersMetadataJob::finished, this, &ShareManager::slotCreateE2eeShareJobFinised);
+    const auto userData = job->userData();
+    Q_ASSERT(userData.sharee);
+    if (!userData.sharee) {
+        qCWarning(lcUserGroupShare) << "missing userData Map in UpdateE2eeShareMetadataJob instance!";
+        emit serverError(-1, tr("Error"));
+        return;
+    }
+    if (statusCode != 200) {
+        emit serverError(statusCode, message);
+    } else {
+        createShare(job->path(), Share::ShareType(userData.sharee->type()), userData.sharee->shareWith(), userData.desiredPermissions, userData.password);
+    }
+}
 }
