@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: 2021 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
+
 #include "notificationhandler.h"
 #include "usermodel.h"
 #include "common/filesystembase.h"
@@ -18,7 +23,6 @@
 #include "tray/unifiedsearchresultslistmodel.h"
 #include "tray/talkreply.h"
 #include "userstatusconnector.h"
-#include "thumbnailjob.h"
 
 #include <QDesktopServices>
 #include <QIcon>
@@ -74,7 +78,7 @@ User::User(AccountStatePtr &account, const bool &isCurrent, QObject *parent)
         this, &User::slotCheckExpiredActivities);
 
     connect(_account.data(), &AccountState::stateChanged,
-            [=]() { if (isConnected()) {slotRefreshImmediately();} });
+            [=, this]() { if (isConnected()) {slotRefreshImmediately();} });
     connect(_account.data(), &AccountState::stateChanged, this, &User::accountStateChanged);
     connect(_account.data(), &AccountState::hasFetchedNavigationApps,
         this, &User::slotRebuildNavigationAppList);
@@ -93,26 +97,45 @@ User::User(AccountStatePtr &account, const bool &isCurrent, QObject *parent)
     connect(_account->account().data(), &Account::capabilitiesChanged, this, &User::slotAccountCapabilitiesChangedRefreshGroupFolders);
 
     connect(_activityModel, &ActivityListModel::sendNotificationRequest, this, &User::slotSendNotificationRequest);
-    
+    connect(_activityModel, &ActivityListModel::showSettingsDialog,
+            Systray::instance(), &Systray::openSettings);
+
     connect(this, &User::sendReplyMessage, this, &User::slotSendReplyMessage);
+
+    connect(_account->account().data(), &Account::userCertificateNeedsMigrationChanged, this, [this] () {
+        auto certificateNeedMigration = Activity{};
+        certificateNeedMigration._type = Activity::OpenSettingsNotificationType;
+        certificateNeedMigration._subject = tr("End-to-end certificate needs to be migrated to a new one");
+        certificateNeedMigration._dateTime = QDateTime::fromString(QDateTime::currentDateTime().toString(), Qt::ISODate);
+        certificateNeedMigration._message = tr("Trigger the migration");
+        certificateNeedMigration._accName = _account->account()->displayName();
+        certificateNeedMigration._id = qHash("migrate-certificate");
+
+        _activityModel->removeActivityFromActivityList(certificateNeedMigration);
+
+        if (_account->account()->e2e()->userCertificateNeedsMigration()) {
+            _activityModel->addNotificationToActivityList(certificateNeedMigration);
+            showDesktopNotification(certificateNeedMigration);
+        }
+    });
 }
 
 void User::checkNotifiedNotifications()
 {
-    // after one hour, clear the gui log notification store
+    // clear the gui log notification store after one hour has passed since the last received notification
     constexpr qint64 clearGuiLogInterval = 60 * 60 * 1000;
     if (_guiLogTimer.elapsed() > clearGuiLogInterval) {
         _notifiedNotifications.clear();
     }
 }
 
-bool User::notificationAlreadyShown(const long notificationId)
+bool User::notificationAlreadyShown(const qint64 notificationId)
 {
     checkNotifiedNotifications();
     return _notifiedNotifications.contains(notificationId);
 }
 
-bool User::canShowNotification(const long notificationId)
+bool User::canShowNotification(const qint64 notificationId)
 {
     ConfigFile cfg;
     return cfg.optionalServerNotifications() &&
@@ -120,17 +143,9 @@ bool User::canShowNotification(const long notificationId)
             !notificationAlreadyShown(notificationId);
 }
 
-void User::checkAndRemoveSeenActivities(const ActivityList &list, const int numTalkNotificationsReceived)
+void User::showDesktopNotification(const QString &title, const QString &message, const qint64 notificationId)
 {
-    if (numTalkNotificationsReceived < _lastTalkNotificationsReceivedCount) {
-        _activityModel->checkAndRemoveSeenActivities(list);
-    }
-    _lastTalkNotificationsReceivedCount = numTalkNotificationsReceived;
-}
-
-void User::showDesktopNotification(const QString &title, const QString &message, const long notificationId)
-{
-    if(!canShowNotification(notificationId)) {
+    if (!canShowNotification(notificationId)) {
         return;
     }
 
@@ -155,7 +170,7 @@ void User::showDesktopNotification(const Activity &activity)
 
 void User::showDesktopNotification(const ActivityList &activityList)
 {
-    const auto subject = tr("%1 notifications").arg(activityList.count());
+    const auto subject = tr("%n notification(s)", nullptr, activityList.count());
     const auto notificationId = -static_cast<int>(qHash(subject));
 
     if (!canShowNotification(notificationId)) {
@@ -174,7 +189,7 @@ void User::showDesktopNotification(const ActivityList &activityList)
 
     Logger::instance()->postGuiLog(subject, message);
 
-    for(const auto &activity : activityList) {
+    for (const auto &activity : activityList) {
         _notifiedNotifications.insert(activity._id);
         _activityModel->addNotificationToActivityList(activity);
     }
@@ -184,7 +199,7 @@ void User::showDesktopTalkNotification(const Activity &activity)
 {
     const auto notificationId = activity._id;
 
-    if (!canShowNotification(notificationId)) {
+    if (!canShowNotification(notificationId) || !ConfigFile().showChatNotifications()) {
         return;
     }
 
@@ -206,43 +221,41 @@ void User::showDesktopTalkNotification(const Activity &activity)
 
 void User::slotBuildNotificationDisplay(const ActivityList &list)
 {
-    const auto talkNotificationsReceivedCount = std::count_if(std::cbegin(list), std::cend(list), [](const auto &activity) {
-        return activity._objectType == QStringLiteral("chat") ||
-            activity._objectType == QStringLiteral("call");
-    });
-    checkAndRemoveSeenActivities(list, talkNotificationsReceivedCount);
-
     ActivityList toNotifyList;
 
-    std::copy_if(list.constBegin(), list.constEnd(), std::back_inserter(toNotifyList), [&](const Activity &activity) {
+    _activityModel->removeOutdatedNotifications(list);
 
-        if (_blacklistedNotifications.contains(activity)) {
-            qCInfo(lcActivity) << "Activity in blacklist, skip";
-            return false;
-        } else if(_notifiedNotifications.contains(activity._id)) {
-            qCInfo(lcActivity) << "Activity already notified, skip";
+    std::copy_if(list.constBegin(), list.constEnd(), std::back_inserter(toNotifyList), [&](const Activity &activity) -> bool {
+        if (!activity._shouldNotify) {
+            qCDebug(lcActivity).nospace() << "No notification should be sent for activity with id=" << activity._id << " objectType=" << activity._objectType;
             return false;
         }
-        if (!activity._shouldNotify) {
-            qCDebug(lcActivity) << "Activity should not be notified";
+
+        if (_notifiedNotifications.contains(activity._id)) {
+            qCInfo(lcActivity).nospace() << "Ignoring already notified activity with id=" << activity._id << " objectType=" << activity._objectType;
             return false;
         }
 
         return true;
     });
 
-    if(toNotifyList.count() > 2) {
-        showDesktopNotification(toNotifyList);
+    if (toNotifyList.isEmpty()) {
         return;
     }
 
-    for (const auto &activity : qAsConst(toNotifyList)) {
+    if (toNotifyList.size() == 1) {
+        const auto &activity = toNotifyList.constFirst();
         if (activity._objectType == QStringLiteral("chat")) {
+            // Talk's "call" type is handled in slotBuildIncomingCallDialogs
             showDesktopTalkNotification(activity);
-        } else {
-            showDesktopNotification(activity);
+            return;
         }
+
+        showDesktopNotification(activity);
+        return;
     }
+
+    showDesktopNotification(toNotifyList);
 }
 
 void User::slotNotificationFetchFinished()
@@ -262,16 +275,18 @@ void User::slotBuildIncomingCallDialogs(const ActivityList &list)
     }
 
     const auto systray = Systray::instance();
+    if (!systray) {
+        qCWarning(lcActivity) << "No systray instance available, can not notify about new calls";
+        return;
+    }
 
-    if(systray) {
-        for(const auto &activity : list) {
-            if (!activity._shouldNotify) {
-                qCDebug(lcActivity) << "Activity should not be notified";
-                continue;
-            }
-
-            systray->createCallDialog(activity, _account);
+    for (const auto &activity : list) {
+        if (!activity._shouldNotify) {
+            qCDebug(lcActivity).nospace() << "No notification should be sent for activity with id=" << activity._id << " objectType=" << activity._objectType;
+            continue;
         }
+
+        systray->createCallDialog(activity, _account);
     }
 }
 
@@ -323,7 +338,7 @@ void User::slotReceivedPushActivity(Account *account)
 void User::slotCheckExpiredActivities()
 {
     const auto errorsList = _activityModel->errorsList();
-    for (const Activity &activity : errorsList) {
+    for (const auto &activity : errorsList) {
         if (activity._expireAtMsecs > 0 && QDateTime::currentDateTime().toMSecsSinceEpoch() >= activity._expireAtMsecs) {
             _activityModel->removeActivityFromActivityList(activity);
         }
@@ -410,7 +425,7 @@ void User::slotRefresh()
     
     if (checkPushNotificationsAreReady()) {
         // we are relying on WebSocket push notifications - ignore refresh attempts from UI
-        slotRefreshActivitiesInitial();
+        slotRefreshActivities();
         _timeSinceLastCheck[_account.data()].invalidate();
         return;
     }
@@ -427,7 +442,7 @@ void User::slotRefresh()
         return;
     }
     if (_account.data() && _account.data()->isConnected()) {
-        slotRefreshActivitiesInitial();
+        slotRefreshActivities();
         slotRefreshNotifications();
         timer.start();
     }
@@ -570,7 +585,7 @@ void User::slotProgressInfo(const QString &folder, const ProgressInfo &progress)
             return;
         const auto &engine = f->syncEngine();
         const auto style = engine.lastLocalDiscoveryStyle();
-        foreach (Activity activity, _activityModel->errorsList()) {
+        for (const auto &activity : _activityModel->errorsList()) {
             if (activity._expireAtMsecs != -1) {
                 // we process expired activities in a different slot
                 continue;
@@ -619,7 +634,7 @@ void User::slotProgressInfo(const QString &folder, const ProgressInfo &progress)
         // We keep track very well of pending conflicts.
         // Inform other components about them.
         QStringList conflicts;
-        foreach (Activity activity, _activityModel->errorsList()) {
+        for (const auto &activity : _activityModel->errorsList()) {
             if (activity._folder == folder
                 && activity._syncFileItemStatus == SyncFileItem::Conflict) {
                 conflicts.append(activity._file);
@@ -851,9 +866,7 @@ void User::processCompletedSyncItem(const Folder *folder, const SyncFileItemPtr
         activity._subject = item->_errorString;
         activity._id = -static_cast<int>(qHash(activity._subject + activity._message));
 
-        if (item->_status == SyncFileItem::Status::FileIgnored) {
-            _activityModel->addIgnoredFileToList(activity);
-        } else {
+        if (item->_status != SyncFileItem::Status::FileIgnored) {
             // add 'protocol error' to activity list
             if (item->_status == SyncFileItem::Status::FileNameInvalid || item->_status == SyncFileItem::Status::FileNameInvalidOnServer) {
                 ActivityLink buttonActivityLink;
@@ -892,7 +905,7 @@ void User::slotItemCompleted(const QString &folder, const SyncFileItemPtr &item)
         return;
     }
 
-    qCDebug(lcActivity) << "Item " << item->_file << " retrieved resulted in " << item->_errorString;
+    qCWarning(lcActivity) << "Item " << item->_file << " retrieved resulted in " << item->_errorString;
     processCompletedSyncItem(folderInstance, item);
 }
 
@@ -1111,6 +1124,11 @@ bool User::isConnected() const
     return (_account->connectionStatus() == AccountState::ConnectionStatus::Connected);
 }
 
+bool User::needsToSignTermsOfService() const
+{
+    return _account->connectionStatus() == AccountState::ConnectionStatus::NeedToSignTermsOfService;
+}
+
 
 bool User::isDesktopNotificationsAllowed() const
 {
@@ -1308,7 +1326,7 @@ QString UserModel::currentUserServer()
 void UserModel::addUser(AccountStatePtr &user, const bool &isCurrent)
 {
     bool containsUser = false;
-    for (const auto &u : qAsConst(_users)) {
+    for (const auto &u : std::as_const(_users)) {
         if (u->account() == user->account()) {
             containsUser = true;
             continue;
@@ -1326,8 +1344,9 @@ void UserModel::addUser(AccountStatePtr &user, const bool &isCurrent)
         });
 
         connect(u, &User::statusChanged, this, [this, row] {
-            emit dataChanged(index(row, 0), index(row, 0), {UserModel::StatusIconRole, 
-			    				    UserModel::StatusEmojiRole,     
+            emit dataChanged(index(row, 0), index(row, 0), {UserModel::StatusRole,
+                                                            UserModel::StatusIconRole,
+                                                            UserModel::StatusEmojiRole,
                                                             UserModel::StatusMessageRole});
         });
         
@@ -1424,7 +1443,7 @@ void UserModel::setCurrentUserId(const int id)
 
     const auto isCurrentUserChanged = !_users[id]->isCurrentUser();
     if (isCurrentUserChanged) {
-        for (const auto user : qAsConst(_users)) {
+        for (const auto user : std::as_const(_users)) {
             user->setCurrentUser(false);
         }
         _users[id]->setCurrentUser(true);
@@ -1519,6 +1538,8 @@ QVariant UserModel::data(const QModelIndex &index, int role) const
         return _users[index.row()]->server();
     } else if (role == ServerHasUserStatusRole) {
         return _users[index.row()]->serverHasUserStatus();
+    } else if (role == StatusRole) {
+        return QVariant::fromValue(_users[index.row()]->status());
     } else if (role == StatusIconRole) {
         return _users[index.row()]->statusIcon();
     } else if (role == StatusEmojiRole) {
@@ -1545,6 +1566,7 @@ QHash<int, QByteArray> UserModel::roleNames() const
     roles[NameRole] = "name";
     roles[ServerRole] = "server";
     roles[ServerHasUserStatusRole] = "serverHasUserStatus";
+    roles[StatusRole] = "status";
     roles[StatusIconRole] = "statusIcon";
     roles[StatusEmojiRole] = "statusEmoji";
     roles[StatusMessageRole] = "statusMessage";
@@ -1618,35 +1640,82 @@ int UserModel::findUserIdForAccount(AccountState *account) const
 }
 /*-------------------------------------------------------------------------------------*/
 
-ImageProvider::ImageProvider()
-    : QQuickImageProvider(QQuickImageProvider::Image)
+class ImageResponse : public QQuickImageResponse
 {
-}
+public:
+    ImageResponse(const QString &id, const QSize &requestedSize, QThreadPool *pool)
+    {
+        Q_UNUSED(pool)
 
-QImage ImageProvider::requestImage(const QString &id, QSize *size, const QSize &requestedSize)
-{
-    Q_UNUSED(size)
-    Q_UNUSED(requestedSize)
+        const auto makeIcon = [](const QString &path) {
+            QImage image(128, 128, QImage::Format_ARGB32);
+            image.fill(Qt::GlobalColor::transparent);
+            QPainter painter(&image);
+            QSvgRenderer renderer(path);
+            renderer.render(&painter);
+            return image;
+        };
 
-    const auto makeIcon = [](const QString &path) {
-        QImage image(128, 128, QImage::Format_ARGB32);
-        image.fill(Qt::GlobalColor::transparent);
-        QPainter painter(&image);
-        QSvgRenderer renderer(path);
-        renderer.render(&painter);
-        return image;
-    };
+        if (id == QLatin1String("fallbackWhite")) {
+            handleDone(makeIcon(QStringLiteral(":/client/theme/white/user.svg")));
+            return;
+        } else if (id == QLatin1String("fallbackBlack")) {
+            handleDone(makeIcon(QStringLiteral(":/client/theme/black/user.svg")));
+            return;
+        }
 
-    if (id == QLatin1String("fallbackWhite")) {
-        return makeIcon(QStringLiteral(":/client/theme/white/user.svg"));
+        if (id.startsWith("user-id=")) {
+            // Format is "image://avatars/user-id=avatar-requested-user/local-user-id:0"
+            const auto userIdsString = id.split('=');
+            const auto userIds = userIdsString.last().split("/local-account:");
+            const auto avatarUserId = userIds.first();
+            const auto accountString = userIds.last();
+            const auto accountState = AccountManager::instance()->account(accountString);
+            Q_ASSERT(accountState);
+            Q_ASSERT(accountState->account());
+            if (!accountState || !accountState->account()) {
+                qCWarning(lcActivity) << "Invalid account:" << accountString;
+                return;
+            }
+
+            const auto account = accountState->account();
+            const auto qnam = account->networkAccessManager();
+
+            QMetaObject::invokeMethod(qnam, [this, requestedSize, avatarUserId, account]() {
+                const auto avatarSize = requestedSize.width() > 0 ? requestedSize.width() : 64;
+                const auto avatarJob = new AvatarJob(account, avatarUserId, avatarSize);
+                connect(avatarJob, &AvatarJob::avatarPixmap, this, [&](const QImage &avatarImg) {
+                    QMetaObject::invokeMethod(this, [this, avatarImg] {
+                        handleDone(AvatarJob::makeCircularAvatar(avatarImg));
+                    });
+                });
+                avatarJob->start();
+            });
+            return;
+        }
+
+        handleDone(UserModel::instance()->avatarById(id.toInt()));
     }
 
-    if (id == QLatin1String("fallbackBlack")) {
-        return makeIcon(QStringLiteral(":/client/theme/black/user.svg"));
+    void handleDone(const QImage &image)
+    {
+        _image = image;
+        emit finished();
     }
 
-    const int uid = id.toInt();
-    return UserModel::instance()->avatarById(uid);
+    QQuickTextureFactory *textureFactory() const override
+    {
+        return QQuickTextureFactory::textureFactoryForImage(_image);
+    }
+
+private:
+    QImage _image;
+};
+
+QQuickImageResponse *ImageProvider::requestImageResponse(const QString &id, const QSize &requestedSize)
+{
+    const auto response = new class ImageResponse(id, requestedSize, &_pool);
+    return response;
 }
 
 /*-------------------------------------------------------------------------------------*/
@@ -1725,3 +1794,4 @@ QHash<int, QByteArray> UserAppsModel::roleNames() const
     return roles;
 }
 }
+
