@@ -149,8 +149,9 @@ QString HttpCredentials::fetchUser()
     return _user;
 }
 
-void HttpCredentials::fetchFromKeychain()
+void HttpCredentials::fetchFromKeychain(const QString &appName)
 {
+    Q_UNUSED(appName)
     _wasFetched = true;
 
     // User must be fetched from config file
