@@ -65,10 +65,28 @@ public:
      */
     void setAutoDelete(bool autoDelete);
 
+    /**
+     * Set application name to be used to search the keychain
+     */
+    void setAppName(const QString &AppName);
+
+#ifdef Q_OS_WIN
+    /**
+     * On Windows the credential keys aren't namespaced properly
+     * by qtkeychain. To work around that we manually add namespacing
+     * to the generated keys. See #6125.
+     * It's safe to do that since the key format is changing for 2.4
+     * anyway to include the account ids. That means old keys can be
+     * migrated to new namespaced keys on windows for 2.4. 
+     */
+    void jobKeyPrependAppName(QString &key);
+#endif
+
 protected:
     QString _serviceName;
     Account *_account = nullptr;
     QString _key;
+    QString _appName;
     bool _insecureFallback = false;
     bool _autoDelete = true;
     bool _keychainMigration = false;
