@@ -1,15 +1,7 @@
 /*
- * Copyright (C) by Olivier Goffart <ogoffart@woboq.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2020 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2014 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "discoveryphase.h"
@@ -82,7 +74,7 @@ void DiscoveryPhase::checkFolderSizeLimit(const QString &path, const std::functi
     connect(propfindJob, &PropfindJob::finishedWithError, this, [=] {
         return completionCallback(false);
     });
-    connect(propfindJob, &PropfindJob::result, this, [=](const QVariantMap &values) {
+    connect(propfindJob, &PropfindJob::result, this, [=, this](const QVariantMap &values) {
         const auto result = values.value(QLatin1String("size")).toLongLong();
         const auto limit = _syncOptions._newBigFolderSizeLimit;
         qCDebug(lcDiscovery) << "Folder size check complete for" << path << "result:" << result << "limit:" << limit;
@@ -180,11 +172,10 @@ QPair<bool, QByteArray> DiscoveryPhase::findAndCancelDeletedJob(const QString &o
             result = true;
             oldEtag = (*it)->_etag;
         } else {
-            if (!(instruction == CSYNC_INSTRUCTION_REMOVE
-                    // re-creation of virtual files count as a delete
-                    || ((*it)->_type == ItemTypeVirtualFile && instruction == CSYNC_INSTRUCTION_NEW)
-                    || ((*it)->_isRestoration && instruction == CSYNC_INSTRUCTION_NEW)))
-            {
+            if (!(instruction == CSYNC_INSTRUCTION_REMOVE ||
+                  instruction == CSYNC_INSTRUCTION_IGNORE ||
+                  ((*it)->_type == ItemTypeVirtualFile && instruction == CSYNC_INSTRUCTION_NEW) ||// re-creation of virtual files count as a delete
+                  ((*it)->_isRestoration && instruction == CSYNC_INSTRUCTION_NEW))) {
                 qCWarning(lcDiscovery) << "ENFORCE(FAILING)" << originalPath;
                 qCWarning(lcDiscovery) << "instruction == CSYNC_INSTRUCTION_REMOVE" << (instruction == CSYNC_INSTRUCTION_REMOVE);
                 qCWarning(lcDiscovery) << "((*it)->_type == ItemTypeVirtualFile && instruction == CSYNC_INSTRUCTION_NEW)"
@@ -225,12 +216,34 @@ void DiscoveryPhase::enqueueDirectoryToDelete(const QString &path, ProcessDirect
     }
 }
 
+void DiscoveryPhase::markPermanentDeletionRequests()
+{
+    // since we don't know in advance which files/directories need to be permanently deleted,
+    // we have to look through all of them at the end of the run
+    for (const auto &originalPath : _permanentDeletionRequests) {
+        const auto it = _deletedItem.find(originalPath);
+        if (it == _deletedItem.end()) {
+            qCWarning(lcDiscovery) << "didn't find an item for" << originalPath << "(yet)";
+            continue;
+        }
+
+        auto item = *it;
+        if (!(item->_instruction == CSYNC_INSTRUCTION_REMOVE || item->_direction == SyncFileItem::Up)) {
+            qCWarning(lcDiscovery) << "will not request permanent deletion for" << originalPath << "as the instruction is not CSYNC_INSTRUCTION_REMOVE, or the direction is not Up";
+            continue;
+        }
+
+        qCInfo(lcDiscovery) << "requested permanent server-side deletion for" << originalPath;
+        item->_wantsPermanentDeletion = true;
+    }
+}
+
 void DiscoveryPhase::startJob(ProcessDirectoryJob *job)
 {
-    ENFORCE(!_currentRootJob);
+    Q_ASSERT(!_currentRootJob);
     connect(this, &DiscoveryPhase::itemDiscovered, this, &DiscoveryPhase::slotItemDiscovered, Qt::UniqueConnection);
     connect(job, &ProcessDirectoryJob::finished, this, [this, job] {
-        ENFORCE(_currentRootJob == sender());
+        Q_ASSERT(_currentRootJob == sender());
         _currentRootJob = nullptr;
         if (job->_dirItem)
             emit itemDiscovered(job->_dirItem);
@@ -242,6 +255,7 @@ void DiscoveryPhase::startJob(ProcessDirectoryJob *job)
             auto nextJob = _queuedDeletedDirectories.take(_queuedDeletedDirectories.firstKey());
             startJob(nextJob);
         } else {
+            markPermanentDeletionRequests();
             emit finished();
         }
     });
@@ -348,6 +362,7 @@ void DiscoverySingleLocalDirectoryJob::run() {
         i.isSymLink = dirent->type == ItemTypeSoftLink;
         i.isVirtualFile = dirent->type == ItemTypeVirtualFile || dirent->type == ItemTypeVirtualFileDownload;
         i.isMetadataMissing = dirent->is_metadata_missing;
+        i.isPermissionsInvalid = dirent->isPermissionsInvalid;
         i.type = dirent->type;
         results.push_back(i);
     }
@@ -373,11 +388,13 @@ DiscoverySingleDirectoryJob::DiscoverySingleDirectoryJob(const AccountPtr &accou
                                                          const QString &path,
                                                          const QString &remoteRootFolderPath,
                                                          const QSet<QString> &topLevelE2eeFolderPaths,
+                                                         SyncFileItem::EncryptionStatus parentEncryptionStatus,
                                                          QObject *parent)
     : QObject(parent)
     , _subPath(remoteRootFolderPath + path)
     , _remoteRootFolderPath(remoteRootFolderPath)
     , _account(account)
+    , _encryptionStatusCurrent{parentEncryptionStatus}
     , _topLevelE2eeFolderPaths(topLevelE2eeFolderPaths)
 {
     Q_ASSERT(!_remoteRootFolderPath.isEmpty());
@@ -393,6 +410,8 @@ void DiscoverySingleDirectoryJob::start()
           << "getlastmodified"
           << "getcontentlength"
           << "getetag"
+          << "quota-available-bytes"
+          << "quota-used-bytes"
           << "http://owncloud.org/ns:size"
           << "http://owncloud.org/ns:id"
           << "http://owncloud.org/ns:fileid"
@@ -400,7 +419,9 @@ void DiscoverySingleDirectoryJob::start()
           << "http://owncloud.org/ns:dDC"
           << "http://owncloud.org/ns:permissions"
           << "http://owncloud.org/ns:checksums"
-          << "http://nextcloud.org/ns:is-encrypted";
+          << "http://nextcloud.org/ns:is-encrypted"
+          << "http://nextcloud.org/ns:metadata-files-live-photo"
+          << "http://nextcloud.org/ns:share-attributes";
 
     if (_isRootPath)
         props << "http://owncloud.org/ns:data-fingerprint";
@@ -550,11 +571,23 @@ static void propertyMapToRemoteInfo(const QMap<QString, QString> &map, RemotePer
         if (property == "lock-token") {
             result.lockToken = value;
         }
+        if (property == "metadata-files-live-photo") {
+            result.livePhotoFile = value;
+            result.isLivePhoto = true;
+        }
     }
 
     if (result.isDirectory && map.contains("size")) {
         result.sizeOfFolder = map.value("size").toInt();
     }
+
+    if (result.isDirectory && map.contains(FolderQuota::usedBytesC)) {
+        result.folderQuota.bytesUsed = map.value(FolderQuota::usedBytesC).toLongLong();
+    }
+
+    if (result.isDirectory && map.contains(FolderQuota::availableBytesC)) {
+        result.folderQuota.bytesAvailable = map.value(FolderQuota::availableBytesC).toLongLong();
+    }
 }
 
 void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(const QString &file, const QMap<QString, QString> &map)
@@ -563,7 +596,7 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(const QString &fi
         // The first entry is for the folder itself, we should process it differently.
         _ignoredFirst = true;
         if (map.contains("permissions")) {
-            auto perm = RemotePermissions::fromServerString(map.value("permissions"),
+            const auto perm = RemotePermissions::fromServerString(map.value("permissions"),
                                                             _account->serverHasMountRootProperty() ? RemotePermissions::MountedPermissionAlgorithm::UseMountRootProperty : RemotePermissions::MountedPermissionAlgorithm::WildGuessMountedSubProperty,
                                                             map);
             emit firstDirectoryPermissions(perm);
@@ -583,17 +616,34 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(const QString &fi
             _fileId = map.value("id").toUtf8();
         }
         if (map.contains("is-encrypted") && map.value("is-encrypted") == QStringLiteral("1")) {
-            _encryptionStatusCurrent = SyncFileItem::EncryptionStatus::Encrypted;
+            _encryptionStatusCurrent = SyncFileItem::EncryptionStatus::EncryptedMigratedV2_0;
             Q_ASSERT(!_fileId.isEmpty());
         }
         if (map.contains("size")) {
             _size = map.value("size").toInt();
         }
+
+        // all folders will contain both
+        if (map.contains(FolderQuota::usedBytesC) && map.contains(FolderQuota::availableBytesC)) {          
+            _folderQuota = {map.value(FolderQuota::usedBytesC).toLongLong(),
+                            map.value(FolderQuota::availableBytesC).toLongLong()};
+            qCWarning(lcDiscovery) << "Setting quota for" << file
+                                   << "bytesUsed:" << _folderQuota.bytesUsed
+                                   << "bytesAvailable:" << _folderQuota.bytesAvailable;
+            emit setfolderQuota(_folderQuota);
+        }
     } else {
         RemoteInfo result;
         int slash = file.lastIndexOf('/');
         result.name = file.mid(slash + 1);
         result.size = -1;
+        if (map.contains(FolderQuota::usedBytesC) && map.contains(FolderQuota::availableBytesC)) {
+            result.folderQuota.bytesAvailable = map.value(FolderQuota::availableBytesC).toLongLong();
+            result.folderQuota.bytesUsed = map.value(FolderQuota::usedBytesC).toLongLong();
+            qCWarning(lcDiscovery) << "Setting quota for" << file
+                                   << "bytesUsed:" <<  result.folderQuota.bytesUsed
+                                   << "bytesAvailable:" << result.folderQuota.bytesAvailable;
+        }
         propertyMapToRemoteInfo(map,
                                 _account->serverHasMountRootProperty() ? RemotePermissions::MountedPermissionAlgorithm::UseMountRootProperty : RemotePermissions::MountedPermissionAlgorithm::WildGuessMountedSubProperty,
                                 result);
@@ -627,36 +677,34 @@ void DiscoverySingleDirectoryJob::lsJobFinishedWithoutErrorSlot()
         emit etag(_firstEtag, QDateTime::fromString(QString::fromUtf8(_lsColJob->responseTimestamp()), Qt::RFC2822Date));
         fetchE2eMetadata();
         return;
-    } else if (isE2eEncrypted() && !_account->capabilities().clientSideEncryptionAvailable()) {
-        emit etag(_firstEtag, QDateTime::fromString(QString::fromUtf8(_lsColJob->responseTimestamp()), Qt::RFC2822Date));
-        emit finished(_results);
     }
     emit etag(_firstEtag, QDateTime::fromString(QString::fromUtf8(_lsColJob->responseTimestamp()), Qt::RFC2822Date));
     emit finished(_results);
     deleteLater();
 }
 
-void DiscoverySingleDirectoryJob::lsJobFinishedWithErrorSlot(QNetworkReply *r)
+void DiscoverySingleDirectoryJob::lsJobFinishedWithErrorSlot(QNetworkReply *reply)
 {
-    const auto contentType = r->header(QNetworkRequest::ContentTypeHeader).toString();
+    const auto contentType = reply->header(QNetworkRequest::ContentTypeHeader).toString();
     const auto invalidContentType = !contentType.contains("application/xml; charset=utf-8") &&
                                     !contentType.contains("application/xml; charset=\"utf-8\"") &&
                                     !contentType.contains("text/xml; charset=utf-8") &&
                                     !contentType.contains("text/xml; charset=\"utf-8\"");
-    const auto httpCode = r->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-    auto msg = r->errorString();
+    const auto httpCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    auto errorString = _lsColJob->errorString();
 
-    qCWarning(lcDiscovery) << "LSCOL job error" << r->errorString() << httpCode << r->error();
+    qCWarning(lcDiscovery) << "LSCOL job error" << reply->errorString() << httpCode << reply->error();
 
-    if (r->error() == QNetworkReply::NoError && invalidContentType) {
-        msg = tr("Server error: PROPFIND reply is not XML formatted!");
+    if (reply->error() == QNetworkReply::NoError && invalidContentType) {
+        errorString = tr("The server returned an unexpected response that couldn’t be read. Please reach out to your server administrator.”");
+        qCWarning(lcDiscovery) << "Server error: PROPFIND reply is not XML formatted!";
     }
 
-    if (r->error() == QNetworkReply::ContentAccessDenied) {
+    if (reply->error() == QNetworkReply::ContentAccessDenied) {
         emit _account->termsOfServiceNeedToBeChecked();
     }
 
-    emit finished(HttpError{ httpCode, msg });
+    emit finished(HttpError{ httpCode, errorString });
     deleteLater();
 }
 
@@ -725,6 +773,9 @@ void DiscoverySingleDirectoryJob::metadataReceived(const QJsonDocument &json, in
         _encryptionStatusRequired = EncryptionStatusEnums::fromEndToEndEncryptionApiVersion(_account->capabilities().clientSideEncryptionVersion());
         _encryptionStatusCurrent = e2EeFolderMetadata->existingMetadataEncryptionStatus();
 
+        Q_ASSERT(_encryptionStatusCurrent != SyncFileItem::EncryptionStatus::Encrypted);
+        Q_ASSERT(_encryptionStatusCurrent != SyncFileItem::EncryptionStatus::NotEncrypted);
+
         const auto encryptedFiles = e2EeFolderMetadata->files();
 
         const auto findEncryptedFile = [=](const QString &name) {
@@ -738,7 +789,7 @@ void DiscoverySingleDirectoryJob::metadataReceived(const QJsonDocument &json, in
             }
         };
 
-        std::transform(std::cbegin(_results), std::cend(_results), std::begin(_results), [=](const RemoteInfo &info) {
+        std::transform(std::cbegin(_results), std::cend(_results), std::begin(_results), [=, this](const RemoteInfo &info) {
             auto result = info;
             const auto encryptedFileInfo = findEncryptedFile(result.name);
             if (encryptedFileInfo) {
