@@ -540,7 +540,9 @@ void PropagateUploadFileNG::slotMoveJobFinished()
 
     if (_item->_etag.isEmpty()) {
         qCWarning(lcPropagateUploadNG) << "Server did not return an ETAG" << _item->_file;
-        abortWithError(SyncFileItem::NormalError, tr("Missing ETag from server"));
+        const auto errorMessage = _item->isDirectory() ? tr("Folder is not accessible on the server.", "server error")
+                                                       : tr("File is not accessible on the server.", "server error");
+        abortWithError(SyncFileItem::NormalError, errorMessage);
         return;
     }
     finalize();
